@@ -43,8 +43,14 @@ import ftrack_api.structure.origin
 import ftrack_api.structure.entity_id
 import ftrack_api.accessor.server
 import ftrack_api._centralized_storage_scenario
+import ftrack_api.logging
 from ftrack_api.logging import LazyLogMessage as L
 
+try:
+    from weakref import WeakMethod
+except ImportError:
+    from ftrack_api._weakref import WeakMethod
+
 
 class SessionAuthentication(requests.auth.AuthBase):
     '''Attach ftrack session authentication information to requests.'''
@@ -251,7 +257,7 @@ class Session(object):
         )
 
         # Register to auto-close session on exit.
-        atexit.register(self.close)
+        atexit.register(WeakMethod(self.close))
 
         self._plugin_paths = plugin_paths
         if self._plugin_paths is None:
@@ -536,7 +542,7 @@ class Session(object):
                 'entity_key': entity.get('id')
             })
 
-        result = self._call(
+        result = self.call(
             [payload]
         )
 
@@ -818,7 +824,7 @@ class Session(object):
         }]
 
         # TODO: When should this execute? How to handle background=True?
-        results = self._call(batch)
+        results = self.call(batch)
 
         # Merge entities into local cache and return merged entities.
         data = []
@@ -1256,7 +1262,7 @@ class Session(object):
 
         # Process batch.
         if batch:
-            result = self._call(batch)
+            result = self.call(batch)
 
             # Clear recorded operations.
             self.recorded_operations.clear()
@@ -1331,7 +1337,7 @@ class Session(object):
 
     def _fetch_server_information(self):
         '''Return server information.'''
-        result = self._call([{'action': 'query_server_information'}])
+        result = self.call([{'action': 'query_server_information'}])
         return result[0]
 
     def _discover_plugins(self, plugin_arguments=None):
@@ -1433,7 +1439,7 @@ class Session(object):
                 'Loading schemas from server due to hash not matching.'
                 'Local: {0!r} != Server: {1!r}', local_schema_hash, server_hash
             ))
-            schemas = self._call([{'action': 'query_schemas'}])[0]
+            schemas = self.call([{'action': 'query_schemas'}])[0]
 
             if schema_cache_path:
                 try:
@@ -1596,8 +1602,24 @@ class Session(object):
             synchronous=True
         )
 
+    @ftrack_api.logging.deprecation_warning(
+        'Session._call is now available as public method Session.call. The '
+        'private method will be removed in version 2.0.'
+    )
     def _call(self, data):
-        '''Make request to server with *data*.'''
+        '''Make request to server with *data* batch describing the actions.
+
+        .. note::
+
+            This private method is now available as public method
+            :meth:`entity_reference`. This alias remains for backwards
+            compatibility, but will be removed in version 2.0.
+
+        '''
+        return self.call(data)
+
+    def call(self, data):
+        '''Make request to server with *data* batch describing the actions.'''
         url = self._server_url + '/api'
         headers = {
             'content-type': 'application/json',
@@ -1624,7 +1646,7 @@ class Session(object):
                 'Server reported error in unexpected format. Raw error was: {0}'
                 .format(response.text)
             )
-            self.logger.error(error_message)
+            self.logger.exception(error_message)
             raise ftrack_api.exception.ServerError(error_message)
 
         else:
@@ -1633,7 +1655,7 @@ class Session(object):
                 error_message = 'Server reported error: {0}({1})'.format(
                     result['exception'], result['content']
                 )
-                self.logger.error(error_message)
+                self.logger.exception(error_message)
                 raise ftrack_api.exception.ServerError(error_message)
 
         return result
@@ -1717,7 +1739,8 @@ class Session(object):
                             value = attribute.get_local_value(item)
 
                     elif entity_attribute_strategy == 'persisted_only':
-                        value = attribute.get_remote_value(item)
+                        if not attribute.computed:
+                            value = attribute.get_remote_value(item)
 
                     if value is not ftrack_api.symbol.NOT_SET:
                         if isinstance(
@@ -1760,11 +1783,16 @@ class Session(object):
 
         return reference
 
+    @ftrack_api.logging.deprecation_warning(
+        'Session._entity_reference is now available as public method '
+        'Session.entity_reference. The private method will be removed '
+        'in version 2.0.'
+    )
     def _entity_reference(self, entity):
         '''Return entity reference that uniquely identifies *entity*.
 
-        Return a mapping containing the __entity_type__ of the entity along with
-        the key, value pairs that make up it's primary key.
+        Return a mapping containing the __entity_type__ of the entity along
+        with the key, value pairs that make up it's primary key.
 
         .. note::
 
@@ -1773,14 +1801,6 @@ class Session(object):
             compatibility, but will be removed in version 2.0.
 
         '''
-        warnings.warn(
-            (
-                "Session._entity_reference is now available as public method "
-                "Session.entity_reference. The private method will be removed "
-                "in version 2.0."
-            ),
-            PendingDeprecationWarning
-        )
         return self.entity_reference(entity)
 
     def decode(self, string):
@@ -2110,6 +2130,10 @@ class Session(object):
 
         return availabilities
 
+    @ftrack_api.logging.deprecation_warning(
+        'Session.delayed_job has been deprecated in favour of session.call. '
+        'Please refer to the release notes for more information.'
+    )
     def delayed_job(self, job_type):
         '''Execute a delayed job on the server, a `ftrack.entity.job.Job` is returned.
 
@@ -2127,7 +2151,7 @@ class Session(object):
         }
 
         try:
-            result = self._call(
+            result = self.call(
                 [operation]
             )[0]
 
@@ -2164,7 +2188,7 @@ class Session(object):
             )
 
         try:
-            result = self._call([operation])
+            result = self.call([operation])
 
         except ftrack_api.exception.ServerError as error:
             # Raise informative error if the action is not supported.
@@ -2266,7 +2290,7 @@ class Session(object):
         }
 
         try:
-            result = self._call([operation])
+            result = self.call([operation])
 
         except ftrack_api.exception.ServerError as error:
             # Raise informative error if the action is not supported.
@@ -2306,7 +2330,7 @@ class Session(object):
         }
 
         try:
-            result = self._call([operation])
+            result = self.call([operation])
 
         except ftrack_api.exception.ServerError as error:
             # Raise informative error if the action is not supported.
@@ -2352,7 +2376,7 @@ class Session(object):
             )
 
         try:
-            self._call(operations)
+            self.call(operations)
 
         except ftrack_api.exception.ServerError as error:
             # Raise informative error if the action is not supported.
@@ -2400,7 +2424,7 @@ class Session(object):
             )
 
         try:
-            self._call(operations)
+            self.call(operations)
         except ftrack_api.exception.ServerError as error:
             # Raise informative error if the action is not supported.
             if 'Invalid action u\'send_review_session_invite\'' in error.message:
